/* Copyright 2001, Todd Lewis <tlewis@mindspring.com>
 * Released under GNU GPL v2. */

/* firewall.c: sets firewall verdict*/

#include "hank.h"
#include "event.h"
#include "packet.h"
#include "paengine.h"
#include "protocol_engine.h"
#include "protocol_lookup.h"
#include "support.h"
#include <stdio.h>
#include <string.h>

#define PRINTF(x...)

static char rcsid[] = "$Id: firewall.c,v 1.6 2001/11/10 01:45:33 tlewis Exp $";

static event_id create_event(void);
static handler_id register_handler(event_id incoming, event_id target, xmlNodePtr m);
static int unregister_handler(handler_id handler);
static int set_handler_target(event_id incoming, handler_id handler, event_id target);
static void handle_event(struct packet *p);

struct event_handler firewall = {
	{
		"event_handler",
		"firewall",
		EHMAJ,
		EHMIN,
		&firewall
	},
	&create_event,
	&register_handler,
	&unregister_handler,
	&set_handler_target,
	&handle_event
};

static u_int32_t verdicts[1024];
static int nv = 1024;

static event_id
create_event(void)
{
	event_id e = --nv;

	if(nv>0){
		verdicts[e] = PA_PACKET_VERDICT_NO_ACTION;
		PRINTF("Returning firewall eid as %i.%i (%i).\n",
			EH_FIREWALL, e, (EH_FIREWALL<<24)|e);
		return((EH_FIREWALL<<24)|e);
	}
	return(0);
}

static handler_id
register_handler(event_id incoming, event_id target, xmlNodePtr m)
{
	xmlNodePtr x;

	PRINTF("Registering firewall as %i.%i.\n", incoming>>24, incoming&0xffffff);
   for(x=m->children;x;x=x->next){
      if(!strcmp((char *)x->name, "text")){
			if(!strcasecmp((char *)x->content, "NO_ACTION")){
				verdicts[incoming&0xffffff] = PA_PACKET_VERDICT_NO_ACTION;
			} else if(!strcasecmp((char *)x->content, "ACCEPT")){
				verdicts[incoming&0xffffff] = PA_PACKET_VERDICT_ACCEPT;
			} else if(!strcasecmp((char *)x->content, "DROP")){
				verdicts[incoming&0xffffff] = PA_PACKET_VERDICT_DROP;
			} else if(!strcasecmp((char *)x->content, "REJECT")){
				verdicts[incoming&0xffffff] = PA_PACKET_VERDICT_REJECT;
			} else {
				return(-1);
			}
      }
   }

	return(0);
}

static int
unregister_handler(handler_id handler)
{
	return(-1);
}

static int
set_handler_target(event_id incoming, handler_id handler, event_id target)
{
	return(-1);
}

static void
handle_event(struct packet *p)
{
	struct event *e;

   for(e=p->q;e;e=e->next){
      if((e->id>>24)==EH_FIREWALL){
			p->verdict |= verdicts[(e->id&0xffffff)];
		}
	}
}

