#include "hank.h"
#include "hdt_blob.h"
#include "packet.h"
#include "paengine.h"
#include "protocol_engine.h"
#include "protocol_lookup.h"
#include "smodule.h"
#include "support.h"
#include "types.h"
#include <fcntl.h>
#include <getopt.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>

static char rcsid[] = "$Id: hank.c,v 1.24 2002/01/04 00:02:45 tlewis Exp $";

#define HANK_VER_MAJ 0
#define HANK_VER_MIN 5
#define HANK_VER_MIN_MIN 1

#define CONFIRM_PAE if(gconf.pae==NULL)\
	gconf.pae=get_pae(gconf.paengine);

#define FUP printf("%i\n", fup++);

static struct configuration *configuration;
static struct collections *collections;
static struct collections cl;

extern char *rcsids[];

/* prototypes */

extern struct smodule *paengine_pcap[];
extern struct smodule *paengine_netfilter[];
extern struct smodule *xml[];
extern struct smodule *plsimple[];

#define P(x) extern struct smodule *x[];
#include "pes"
#undef P

/* cheats */
extern struct protocol_lookup pls;
extern struct smodule_juggler sj;

extern struct memcon mmapmc;
extern struct memcon mallocmc;

struct memcon *mc = &mmapmc;

#define PRINTF(x...)

lt proto_table;
event_id suigeneris;
static int np;

static struct gconf {
	char *configfile;
	char *serverroot;
	char *logdir;
	char *moduledir;
	int npackets;
	char *paengine;
	struct paengine *pae;
	int quiet;
	char *rulesfile;
} gconf = {
	"./rules.xml",
	".",
	"/var/lib/hank/log",
	"/var/lib/hank/modules",
	-1,
	"pcap",
	NULL,
	0,
	NULL
};

static void
usage(void)
{
	printf("Usage: hank\n"
   /* "	[-c config-file (default=%s)]\n" */
   "	[-d serverroot (default=%s)]\n"
   "	[-D (background mode)]\n"
   "	[-h (help)]\n"
   "	[-l log-dir (default=%s)]\n"
   "	[-m module-dir (default=%s)]\n"
   "	[-n num-packets (-1=infinity, default=%i)]\n"
   "	[-p paengine (default=%s)]\n"
   "	[-P paengine-argument]\n"
   "	[-q (quiet)]\n"
   "	[-r rule-file   (default=%s)]\n"
   "	[-t (test validity of config file & rules)]\n"
   "	[-v (version)]\n"
   "	[-V (version & build parameters)]\n",
	/* gconf.configfile, */
	gconf.serverroot,
	gconf.logdir,
	gconf.moduledir,
	gconf.npackets,
	gconf.paengine,
	gconf.rulesfile);

}

static void
version(void)
{
	printf("Hank version: %i.%i.%i, compiled %s, %s\n", 
		HANK_VER_MAJ, HANK_VER_MIN, HANK_VER_MIN_MIN, __DATE__, __TIME__);
	printf("Compiled with: gcc version %s\n", __VERSION__);
}

static void
validate(void)
{
	fprintf(stderr, "Validation not yet implemented; apologies.\n");
}

static void
huphandler(int s)
{
	signal(SIGHUP, huphandler);
	printf("%i\n", np);
}

static void
dump_rcsids(void)
{
	int i;

	printf("RCS ids:\n");
	for(i=0;rcsids[i]!=NULL;i++){
		printf("     %s\n", rcsids[i]);
	}
}

static void
daemonize(void)
{
	int i;

	/* Unix Programming FAQ (v1.37) */
	/* 1.7 How do I get my program to act like a daemon? */
	if((fork()!=0))
		exit(0);

	if((setsid()==-1)){
		fprintf(stderr, "ERROR: Can't setsid(); exiting.\n");
		exit(-1);
	}

	if((fork()!=0))
		exit(0);

	if(chdir(gconf.serverroot)){
		fprintf(stderr, "ERROR: Can't chdir(%s); exiting.\n", gconf.serverroot);
		exit(-1);
	}

	umask(0);

	for(i=0;i<3;i++){
		if((close(i)==-1)){
			fprintf(stderr, "ERROR: Can't close file descriptor %i.  Exiting.\n", i);
			exit(-1);
		}
	}

	if((open("/dev/null", O_RDONLY)==-1)){
			fprintf(stderr, "ERROR: Can't open /dev/null for reading.  Exiting.\n");
			exit(-1);
	}

	if((open("/dev/null", O_WRONLY)==-1)){
			fprintf(stderr, "ERROR: Can't open /dev/null for writing.  Exiting.\n");
			exit(-1);
	}

	if((open("/dev/null", O_WRONLY)==-1)){
			fprintf(stderr, "ERROR: Can't open /dev/null for writing.  Exiting.\n");
			exit(-1);
	}

}

static struct paengine *
get_pae(char *pae_name)
{
	struct smodule *sm;
	struct paengine *pae;
	int i;

	sm=sj.get_smodule(cl.paengines, pae_name);
	if(sm==NULL){
		fprintf(stderr, "get_smodule() for paengine failed on collection %i; exiting.\n", cl.paengines);
		exit(-1);
	}
	pae=sm->interface;
	if((i=pae->init())!=0){
		fprintf(stderr, "paengine->init() returned %i; exiting.\n", i);
		exit(i);
	}

	return(pae);
}

/* public functions */
int
main(int argc, char* argv[])
{
	struct smodule **statics[16];
	struct smodule **smc;
	struct smodule *sm;
	struct protocol_engine *pe;
	int ch, i, j, k, l;
	struct packet *p;
	u_int32_t proto;
	struct stat buf;
	unsigned long ab, ae, prb, pre;

	collections=&cl;

	i=0;
#define P(x) statics[i++] = x;
#include "pes"
#undef P
	statics[i]=NULL;

	proto_table=pls.create_lt();
	for(i=0;statics[i]!=NULL;i++){
		smc=statics[i];
		for(j=0;smc[j]!=NULL;j++){
			sm=smc[j];
			pe=sm->interface;
			for(k=0;pe->protocols[k]!=0;k++){
				proto=pe->protocols[k];
				l=pls.reg_proto(proto_table, pe, proto);
				if(l!=0){
					fprintf(stderr, "Registration of proto %u failed!  Exiting.\n", proto);
					exit(-1);
				}
			}
		}
	}

	statics[0]=paengine_pcap;
	statics[1]=paengine_netfilter;
	statics[2]=NULL;

	cl.paengines=sj.scan_collection("paengine", statics, NULL, PAVMAJ, PAVMIN);
	if(cl.paengines<0){
		fprintf(stderr, "scan_collections() for paengine failed(%i); exiting.\n",
			cl.paengines);
		exit(-1);
	}

	while ((ch = getopt(argc, argv, "c:d:Dhl:m:n:p:P:qr:tvV")) != EOF) {
		switch (ch) {
			case 'c':
				gconf.configfile=optarg;
				break;

			case 'd':
				gconf.serverroot=optarg;
				break;

			case 'D':
				daemonize();
				break;

			case 'h':
				usage();
				exit(0);
				break; /* never reached */

			case 'l':
				fprintf(stderr, "WARNING: logging not implemented.\n");
				gconf.logdir=optarg;
				break;

			case 'm':
				gconf.moduledir=optarg;
				break;

			case 'n':
				gconf.npackets=atoi(optarg);
				break;

			case 'p':
				gconf.paengine=optarg;
				break;

			case 'P':

				CONFIRM_PAE

				if((i=gconf.pae->configure(optarg))!=0){
					fprintf(stderr, "paengine->configure(%s) for paengine \"%s\" returned %i; exiting.\n", optarg, gconf.pae->s.name, i);
					exit(i);
				}
				break;

			case 'q':
				gconf.quiet=1;
				break;

			case 'r':
				gconf.rulesfile=optarg;
				break;

			case 't':
				validate();
				exit(0);
				break;

			case 'v':
				version();
				exit(0);
				break;

			case 'V':
				version();
				dump_rcsids();
				exit(0);
				break;

			default:
				fprintf(stderr, "AIEE!  Unrecognized argument: \"%s\".  Exiting.\n", optarg);
				exit(-1);
		}
	}

	chdir(gconf.serverroot);

	CONFIRM_PAE

	PRINTF("Establishing rules...\n");

	establish_rules(gconf.configfile);

	PRINTF("Original event is %i.%i.  Beginning loop.\n", (suigeneris>>24), (suigeneris&0xffffff));

	signal(SIGHUP, huphandler);
	np=0;
	build_blobs();

	unlink("./hank_stop");

	np=0;
	while((gconf.npackets < 0) || (np < gconf.npackets)){

		np++;
		rdtscl(ab);
		if((p=gconf.pae->get_packet(suigeneris))==NULL){
			PRINTF("Finishing.\n");
			break;
		}

		i=p->encaps->length;
		rdtscl(ae);

		rdtscl(prb);
		handle_events(p);
		rdtscl(pre);

		gconf.pae->dispose_packet(p);


		printf("    (Packet %i : %lu cycles, %i bytes, acquisition %lu cycles)\n", np, (pre-prb), i, (ae-ab));

		if(!(stat("./hank_stop", &buf)))
			exit(0);
	}

	exit(0);
}
