#include "hdt_ipv4.h"
#include "hank.h"
#include "memcon.h"
#include "support.h"
#include <string.h>

extern struct memcon *mc;

static char rcsid[] = "$Id: hdt_ipv4.c,v 1.5 2001/11/10 01:45:33 tlewis Exp $";

/* This needs to contain a single registered op. */
/* needs to be ref-counted. */
struct hdt_ipv4_prim {
	/* add a ref count */
	u_int32_t val;
	prim_id prid;
	struct hdt_ipv4_prim *next;
};

/* This needs to contain all of the registered ops in the set. */
/* Has one list for each op. */
struct hdt_ipv4_set {
	struct memcon_context *c;
	struct hdt_ipv4_prim *subnet;
	/* add a list of prims */
};

static struct hdt_ipv4_set *sets[16384];
static int hdt_ipv4_sets_offset=0;

hdt_set 
hdt_ipv4_create(void)
{
	hdt_set o = hdt_ipv4_sets_offset++;
	struct memcon_context *c;
	struct hdt_ipv4_set *s;

	if((unsigned int)o >= sizeof(sets)) /* out of room! */
		return(-1);

	/* create a new memcon */
	c=mc->create(0, "hdt_ipv4");
	if(c==NULL)
		exit(-1);

	/* allocate a new set */
	if((s=c->eng->alloc(c, sizeof(*s)))==NULL)
		exit(-1);

	/* populate the set */
	memset(s, 0, sizeof(*s));
	s->c=c;

	/* insert the set */
	sets[o]=s;

	/* return the offset */
	return(o);
}

void 
hdt_ipv4_destroy(hdt_set set)
{
	struct hdt_ipv4_set *s = sets[set];

	/* free all of the plist's */
	s->c->eng->destroy(s->c);
	sets[set]=NULL;
}

hdt_prim 
hdt_ipv4_add(hdt_set set, prim_id prid, char *op, char *arg)
{
	struct hdt_ipv4_set *s = sets[set];
	struct hdt_ipv4_prim *p;
	struct hdt_ipv4_two_prim *r;
	char *a[2];

	/* TODO: detect duplicates */
	if(!strcmp(op, "<")){
		/* subnet of */
		if((p=s->c->eng->alloc(s->c, sizeof(*p)))==NULL)
			exit(-1);
		p->prid=prid;
		p->val=atoi(arg);
		p->next=s->equals;
		s->equals=p;

	} else {
		return(-1);
	}
	/* need to return list offset */
	return(0);
}

void 
hdt_ipv4_del(hdt_set set, hdt_prim prim)
{
	/* need a list of prims in hdt_ipv4_set */
	/* not implemented */
}

int
hdt_ipv4_match(hdt_set set, hdt_prim *prims, u_int32_t arg, unsigned char *rbuf)
{
	struct hdt_ipv4_prim *i;
	struct hdt_ipv4_two_prim *r;
	struct hdt_ipv4_set *s = sets[set];

	/* equals */
	for(i=s->equals; r; r=r->next){
		if(i->val==arg)
			rbuf[i->prid]=1;
	}

	return(0);
}

