#include "hdt_ipv4_addr.h"
#include "hank.h"
#include "memcon.h"
#include "support.h"
#include <string.h>
#include <stdio.h>

/* A simple btree ip-style address matching data type implementation.
 * All addresses are stored and handled in network byte order.
 * However, the tree is true, and so traversing the tree is done after
 * converting the address to host order.
 */

#define SIZE_T u_int32_t

static char rcsid[] = "$Id: hdt_ipv4_addr.c,v 1.14 2001/12/08 00:04:48 tlewis Exp $";

extern struct memcon *mc;

#if 1
#define PRINTF(x...) 
#else
#define PRINTF(x...) printf(##x)
#endif

struct ipv4_network {
	u_int32_t addr;
	char mask; /* max of 32 */
};

/* This needs to contain a single registered op. */
/* needs to be ref-counted. */
struct hdt_ipv4_prim {
	struct ipv4_network *n;
	prim_id prid;
	struct hdt_ipv4_prim *next;
};

struct ip_btree {
	struct hdt_ipv4_prim *networks;
	struct ip_btree *zero;
	struct ip_btree *one;
};

static struct ip_btree *sets[16384];
static int hdt_ipv4_sets_offset=16383;

/* Syntax: "addr(/mask)", e.g., "192.168/16" (default mask=/32) */
static struct ipv4_network *
hs_convert_ip_net(char *arg)
{
	char *mask;
	char net[16]; /* aaa.bbb.ccc.ddd<null> */
	struct ipv4_network *n;
	int i, j, hasnum=0;
	SIZE_T s;

	if((n=malloc(sizeof(*n)))==NULL)
		exit(-1);

	if(!strcmp(arg, "any")){
		n->mask=0;
		n->addr=0;
		return(n);
	}

	mask=strstr(arg, "/");
	if ((mask>=&arg[strlen(arg)-1])||(mask==NULL)) {
		n->mask=32;
	} else {
		*mask=0;
		mask++;
		n->mask=atoi(mask);
		/* printf("Rendered mask %s as %i.\n", mask, n->mask); */
		if(n->mask>32)
			goto failure;
	}

	strncpy(net, arg, sizeof(net));
	j=0;
	for(i=0;net[i]!=0;i++){
		if(net[i]=='.')
			j++;
		if((net[i]>='0') && (net[i]<='9'))
			hasnum=1;
	}

	if(hasnum==0){
		goto failure;
	}

	for(i=3-j;i;i--){
		s=strlen(net);
		if(s<13){
			net[s]='.';
			net[s+1]='0';
			net[s+2]=0;
		}
	}

	n->addr=inet_addr(net);

	if(n->addr!=(in_addr_t)~0)
		return(n);

failure:
	free(n);
	return(NULL);
}

hdt_set 
hdt_ipv4_addr_create(void)
{
	hdt_set o = hdt_ipv4_sets_offset;
	struct memcon_context *c;
	struct ip_btree *s;

	if(o < 0) /* out of room! */
		return(-1);

	hdt_ipv4_sets_offset--;

	/* create a new memcon */
	c=mc->create(0, "hdt_ipv4");
	if(c==NULL)
		exit(-1);

	/* allocate a new set */
	if((s=malloc(sizeof(*s)))==NULL)
		exit(-1);

	/* populate the set */
	memset(s, 0, sizeof(*s));

	/* insert the set */
	sets[o]=s;

	/* return the offset */
	return(o);
}

void 
hdt_ipv4_addr_destroy(hdt_set set)
{
	/* Uhh, this isn't well-supported. */
}

hdt_prim 
hdt_ipv4_addr_add(hdt_set set, prim_id prid, char *op, char *arg)
{
	struct ip_btree *s = sets[set];
	struct hdt_ipv4_prim *p;
	int i;

	PRINTF("Adding ipv4 prim...\n");

	if(!strcmp(op, "<")){
		/* subnet of */
		if((p=malloc(sizeof(*p)))==NULL)
			exit(-1);

		p->prid=prid;
		p->n=hs_convert_ip_net(arg);
		if(p->n->mask == 0){
			/* should handle always-trues here */
		}

		/* find appropriate s */
		for(i=0; i < p->n->mask; i++){
			if( (ntohl(p->n->addr)>>i) & 1){
				/* two roads diverged in a yellow wood */
				if(s->one==NULL){
					/* printf("Allocating %i bytes.\n", sizeof(*(s->one))); */
					if((s->one=malloc(sizeof(*(s->one))))==NULL)
						exit(-1);
					memset(s->one, 0, sizeof(*s->one));
				}
				PRINTF("Loop! 1, %i; ", i);
				s=s->one;
			} else {
				/* sorry I could not travel both */
				if(s->zero==NULL){
					/* printf("Allocating %i bytes.\n", sizeof(*(s->zero))); */
					if((s->zero=malloc(sizeof(*(s->zero))))==NULL)
						exit(-1);
					memset(s->zero, 0, sizeof(*s->zero));
				}
				PRINTF("Loop! 0, %i; ", i);
				s=s->zero;
			}
		}

		p->next=s->networks;
		s->networks=p;
		PRINTF("\nIPV4: Got to the end!  New is %p, next is %p\n", p, p->next);

	} else {
		/* unsupported operation */
		return(-1);
	}

	/* need to return list offset */
	return(0);
}

void 
hdt_ipv4_addr_del(hdt_set set, hdt_prim prim)
{
	/* need a list of prims in hdt_ipv4_set */
	/* not implemented */
}

int
hdt_ipv4_addr_match(hdt_set set, hdt_prim *prims, u_int32_t arg, unsigned char *rbuf, char newbits)
{
	struct hdt_ipv4_prim *n;
	struct ip_btree *s = sets[set];
	int i=0;
	u_int32_t m;
	unsigned long b, e;
	int j=0;

	rdtscl(b);

	if(prims!=NULL){
		fprintf(stderr, "AIEE!  Partial matches not supported for ipv4_addr hdt.\n");
		return(-1);
	}

	while(s){
		for(n=s->networks;n;n=n->next){
			j++;

			if(n->n->mask)
				m = htonl( ((u_int32_t)~0) << (32 - n->n->mask) );
			else
				m=0;

			PRINTF("Set mask to %u (%u << (32 - %i))\n", m, (u_int32_t)~0, n->n->mask);
			/* PRINTF("FYI: /1 is %u (%u << (32 - %i))\n", 
				( ((u_int32_t)~0) << (32 - 1) ), (u_int32_t)~0, 1); */

			if( ((arg) & m) == ((n->n->addr) & m) ) {
				rbuf[n->prid] |= newbits;
				PRINTF("IPV4: %i.%i.%i.%i < %i.%i.%i.%i/%i (%u)\n",
					arg & 0xff, arg>> 8 & 0xff, arg>>16 & 0xff, arg>>24 & 0xff,
					n->n->addr & 0xff, n->n->addr>> 8 & 0xff, 
					n->n->addr>>16 & 0xff, n->n->addr>>24 & 0xff, 
					n->n->mask, m);
			} else {
				PRINTF("IPV4: %i.%i.%i.%i !< %i.%i.%i.%i/%i (%u)\n",
					arg & 0xff, arg>> 8 & 0xff, arg>>16 & 0xff, arg>>24 & 0xff,
					n->n->addr & 0xff, n->n->addr>> 8 & 0xff, 
					n->n->addr>>16 & 0xff, n->n->addr>>24 & 0xff, 
					n->n->mask, m);
			}

			PRINTF("Going to next network at %p.\n", n->next);
		}

		if( (ntohl(arg) >> i++) & 1){
			s=s->one;
			PRINTF("next, 1, %p\n", s);
		} else {
			s=s->zero;
			PRINTF("next, 0, %p\n", s);
		}
	}
	rdtscl(e);
	printf("ipv4 addr match took %ul cycles for %i iterations.\n", (e-b), j);

	return(0);
}

char *
hdt_ipv4_addr_report(u_int32_t val, struct memcon_context *c)
{
	char *r;

	if((r=c->eng->alloc(c, 17))==NULL)
		exit(-1);

	snprintf(r, 16, "%i.%i.%i.%i",
		val     & 0xff, val>> 8 & 0xff, 
		val>>16 & 0xff, val>>24 & 0xff);

	return(r);
}

