#include "protocol_engine.h"
#include "hank.h"
#include "protocol_lookup.h"
#include "support.h"
#include <stdlib.h>
#include <stdio.h>

static char rcsid[] = "$Id: protocol_lookup_simple.c,v 1.5 2001/11/10 01:45:33 tlewis Exp $";

static lt create_lt(void);
static int reg_proto(lt table, struct protocol_engine *pe, u_int32_t protocol);
static int unreg_proto(lt table, u_int32_t protocol);
static int recompute(lt table);
static struct protocol_engine *find_pe_num(lt table, u_int32_t protocol);
static struct protocol_engine *find_pe_name(lt table, char *name);
static void destroy_lt(lt table);

#define PRINTF(x...)

struct protocol_lookup pls= {
	{
		"protocol_lookup",
		"simple",
		PLVMAJ,
		PLVMIN,
		&pls
	},
	&create_lt,
	&reg_proto,
	&unreg_proto,
	&recompute,
	&find_pe_num,
	&find_pe_name,
	&destroy_lt
};

#define STATIC_MODULE_NAME pl_simple

#ifdef ENABLE_DYN_MODULES
struct smodule * paengine [2] = {
#else  /*ENABLE_DYN_MODULES*/
struct smodule * STATIC_MODULE_NAME [2] =  {
#endif /*ENABLE_DYN_MODULES*/
	&(pls.s),
	NULL
};

struct proto_e {
	const char *name;
	struct proto_e *next;
	u_int32_t proto;
	struct protocol_engine *pe;
};

static struct proto_e *
create_pre(void)
{
	struct proto_e *e;

	e=malloc(sizeof(*e));
	if(e==NULL)
		/* FIXME */
		exit(-1);

	return(e);
}

static lt 
create_lt(void)
{
	struct proto_e *e;

	e=create_pre();
	e->next=NULL;
	e->proto=0;
	e->pe=NULL;

	return(e);
}

static int 
reg_proto(lt table, struct protocol_engine *pe, u_int32_t protocol)
{
	struct proto_e *e, *f, *l;

	l=table;

	e=create_pre();
	for(f=l;f->next;f=f->next)
		;

	e->next=NULL;
	f->next=e;

	e->proto=protocol;
	e->pe=pe;
	e->name=pe->s.name;
	PRINTF("Registering protocol: %s %s.%u\n", 
		e->name, find_by_num(protocol_names, protocol>>24), (u_int16_t)protocol);
	return(0);
}

static int 
unreg_proto(lt table, u_int32_t protocol)
{
	struct proto_e *m=table, *n;

	for(n=table;n->next;n=n->next){
		if(n->proto==protocol){
			m->next=n->next;
			free(n);
			return(0);
		}
		m=n;
	}
	return(-1);
}

static int 
recompute(lt table)
{
	if(table==NULL)
		return(0);
	return(0);
}

static struct protocol_engine *
find_pe_num(lt table, u_int32_t protocol)
{
	struct proto_e *n=table;

	for(n=n->next;n&&n->next;n=n->next){
		if(n->proto==protocol){
			return(n->pe);
		}
	}
	return(NULL);
}

static struct protocol_engine *
find_pe_name(lt table, char *name)
{
	struct proto_e *n=table;

	for(n=n->next;n;n=n->next){
		if(!strcmp(n->name,name)){
			return(n->pe);
		}
	}
	PRINTF("AIEE!  No match found for %s!\n", name);
	return(NULL);
}

static void 
destroy_lt(lt table)
{
	struct proto_e *m, *n;

	for(n=table;n;){
		m=n->next;
		free(n);
		n=m;
	}
}

