#include <support.h>
#include "hank.h"

static char rcsid[] = "$Id: stream.c,v 1.5 2001/11/10 01:45:33 tlewis Exp $";

/* the internal stream representation */
struct stream {
	u_int32_t saddr;
	u_int32_t daddr;
	u_int16_t sport;
	u_int16_t dport;
	void *s;
	struct stream *next;
};

/* the hash table structure */
struct stream_col {
	u_int32_t randomizer;
	u_int32_t mask;
	int (*hash)(u_int32_t seed,
		u_int32_t saddr, u_int32_t daddr, 
		u_int16_t sport, u_int16_t dport);
	struct stream *streams; /* array of len (mask+1) */
};

struct stream_col collections[16];
int nc=16;

/* private routines */

static unsigned int
hash_8(u_int32_t seed, u_int32_t saddr, u_int32_t daddr, 
	u_int16_t sport, u_int16_t dport)
{
	u_int8_t r;
	u_int32_t a, b;
	u_int16_t c, d;

	if(saddr<daddr){
		a=saddr;
		b=daddr;
	} else {
		b=saddr;
		a=daddr;
	}

	if(sport<dport){
		c=sport;
		d=dport;
	} else {
		d=sport;
		c=dport;
	}

	r=seed;

	r ^= a;
	r += (a>>8);
	r -= (a>>16);
	r ^= (a>>24);

	r ^= b;
	r -= (b>>8);
	r += (b>>16);
	r ^= (b>>24);

	r ^= c;
	r ^= (c>>8);

	r ^= d;
	r ^= (d>>8);

	return(r);
}

static unsigned int
hash_16(u_int32_t seed, u_int32_t saddr, u_int32_t daddr, 
	u_int16_t sport, u_int16_t dport)
{
	u_int16_t r;
	u_int32_t a, b;
	u_int16_t c, d;

	if(saddr<daddr){
		a=saddr;
		b=daddr;
	} else {
		b=saddr;
		a=daddr;
	}

	if(sport<dport){
		c=sport;
		d=dport;
	} else {
		d=sport;
		c=dport;
	}

	r=seed;

	r -= a;
	r ^= (a>>16);

	r += b;
	r ^= (b>>16);

	r -= c;
	r ^= d;

	return(r);
}

unsigned int
hash_32(u_int32_t seed, u_int32_t saddr, u_int32_t daddr, 
	u_int16_t sport, u_int16_t dport)
{
	u_int32_t r;
	u_int32_t a, b;
	u_int16_t c, d;

	if(saddr<daddr){
		a=saddr;
		b=daddr;
	} else {
		b=saddr;
		a=daddr;
	}

	if(sport<dport){
		c=sport;
		d=dport;
	} else {
		d=sport;
		c=dport;
	}

	r=seed;
	r -= a;
	r ^= b;
	r ^= (c | (d << 16) );

	return(r);
}

/* public routines */

/* typedef u_int32_t streams_col_t; */
/* size here refers to the bitsize of the collection.  E.g., "16"
   will allow you to handle 65535 streams. */

streams_col_t 
create_streams(int size)
{
	int i;
	struct stream_col *c;
	struct timeval t;

	gettimeofday(&t, NULL);

	i = nc-- - 1;
	c=&collections[i];

	c->mask = 0~ >> ((sizeof(0~)*8)-size);

	/* clever, no? */
	c->randomizer = srand((unsigned int)&c ^ t.tv_sec + t.tv_usec);

	if((c->streams=malloc((c->mask+1)*sizeof(struct stream *)))==NULL)
		exit(-1);
	memset(c->stream, 0, ((c->mask+1)*sizeof(struct stream *)));

	if(size <= 8)
		c->hash=&hash_8;
	else if(size <= 16)
		c->hash=&hash_16;
	else
		c->hash=&hash_32;

	return(i);
}

int 
destroy_streams(streams_col_t c)
{
	return(-1);
}

/* returns 0 on success, other on failure */
int register_stream(streams_col_t col, void *stream,
	u_int32_t saddr, u_int32_t daddr, 
	u_int16_t sport, u_int16_t dport)
{
	/* allocate new stream */
	/* populate new stream */
	/* compute hash */
	/* insert into collection */
	return(-1);
}

/* returns NULL on failure */
void *
get_stream(streams_col_t col, 
	u_int32_t saddr, u_int32_t daddr, 
	u_int16_t sport, u_int16_t dport)
{
	struct stream_col *c = &collections[col];
	struct stream *s;
	unsigned int i;
	u_int32_t a, b;
	u_int16_t c, d;

	if(saddr<daddr){
		a=saddr;
		b=daddr;
	} else {
		b=saddr;
		a=daddr;
	}

	if(sport<dport){
		c=sport;
		d=dport;
	} else {
		d=sport;
		c=dport;
	}

	i=c->hash(c->rand, a, b, c, d);
	for(s=c->streams[i];s;s=s->next){
	}

	return(NULL);
}

/* returns 0 on success, other on failure */
int unregister_stream(streams_col_t col, 
	u_int32_t saddr, u_int32_t daddr, 
	u_int16_t sport, u_int16_t dport)
{
	return(-1);
}
