#include "protocols.h"
#include "hank.h"
#include "support.h"
#include "packet.h"
#include <string.h>
#include <stdio.h>

#define SIZE_T u_int32_t

static char rcsid[] = "$Id: support.c,v 1.8 2001/11/10 01:45:33 tlewis Exp $";

#define X(a,b) {#a, b},
struct name_table protocol_names[] = {
#include "protocols.X"
   {"", 0}
};
#undef X

/* Returns a u_int32_t[2] with min and max     */
/* Syntax: "min(-max)", e.g., "0-1023" or "23" */
/*         (if a single int, then min=max=int) */
u_int32_t *
hs_convert_ranges(char *arg, struct memcon_context *m)
{
	if(!m)
		return(NULL);
	if(!arg)
		return(NULL);

	return(NULL);
}

void 
bit_dump(u_int32_t n, char *buf, SIZE_T size)
{
	unsigned int i;

	if(size<(8*sizeof(n))){
		if(size>0)
			buf[0]=0;
		return;
	}

	buf[size]=0;
	for(i=1;i<=size;i++){
		if((n>>i)&1)
			buf[size-i]='1';
		else
			buf[size-i]='0';
	}
}

/* Splits "key=value" into two tokens and returns them. */
void 
split_kv(char *string, char *kv[2], char delim)
{
	SIZE_T s, i;

	kv[0]=string;
	kv[1]=0;

	s=strlen(string);
	for(i=0;i<(s-1);i++){
		if(string[i]==delim){
			string[i]=0;
			kv[1]=&(string[i+1]);
			return;
		}
	}
	if(string[s]==delim)
		string[s]=0;

	return;
}

const char* 
find_by_num (struct name_table *t, u_int32_t num   )
{
	struct name_table *p;

	for(p=t;p->num!=0;p++){
		if(p->num==num)
			return(p->name);
	}
	return(NULL);
}

u_int32_t   
find_by_name(struct name_table *t, const char *name)
{
	struct name_table *p;

	if(name==NULL)
		return(0);

	for(p=t;p->num!=0;p++){
		if(!strcmp(p->name, name))
			return(p->num);
	}
	return(0);
}


const char *
find_proto_name(u_int32_t protocol)
{
	return(find_by_num(protocol_names, protocol));
}

const u_int32_t 
find_proto_num(const char *name)
{
	return(find_by_name(protocol_names, name));
}

u_int16_t 
bs16(u_int16_t i)
{
	union {
		u_int16_t x;
		u_int8_t  y[2];
	} a, b;

	a.x=i;
	b.y[0]=a.y[1];
	b.y[1]=a.y[0];
	return(b.x);
}

u_int32_t 
bs32(u_int32_t i)
{
	union {
		u_int32_t x;
		u_int8_t  y[4];
	} a, b;


	a.x=i;
	b.y[0]=a.y[3];
	b.y[1]=a.y[2];
	b.y[2]=a.y[1];
	b.y[3]=a.y[0];

	return(b.x);
}

__inline__ char
test_int_field(struct int_field_test *t, u_int32_t val)
{
#if 0
	printf("val %i op %c test %i\n", val, t->op, t->a);
#endif 

	switch(t->op){
		case '%':
			return(val%t->a);
		case '>':
			return(val>t->a);
		case '<':
			return(val<t->a);
		case '&':
			return(val&t->a);
		case '=':
			return(val==t->a);
		case ':':
			return( (val > t->a ) && (val < t->b) );
	}
	/* FIXME: log */
	printf("ERROR: NO OP FOUND: op=%c, val=%i, a=%i, b=%i\n", t->op, val, t->a, t->b);
	return(-1);
}

void
append_encap(struct packet *p, u_int32_t protocol, SIZE_T length, unsigned char *payload)
{
	struct capsule *e, *f;

	e=p->encaps;
   while(e->next)
      e=e->next;

	/* FIXME: log */
	if((f=p->c->eng->alloc(p->c, sizeof(struct capsule)))==NULL)
		exit(-1);

   f->matches=NULL;
   f->protocol=protocol;
   f->payload=payload;
   f->length=length;
   f->payload_length=0;
	f->payload_protocol=0;
	f->payload=NULL;
   f->pe=NULL;
   f->next=NULL;
   f->parent=p;
	e->next=f;

}

xmlNodePtr 
get_real_xml_node(xmlNodePtr x)
{
	xmlNodePtr y=x;

	while(y){
		if(strcmp((const char *)y->name, "text"))
			return(y);
		y=y->next;
	}
	return(NULL);
}

unsigned int 
plist_add(struct plist *list, void* pointer)
{
	SIZE_T t;
	void *v;

	if(list->in_use==list->array_size){
		/* *2 */
		t=(list->array_size*2);
		v=malloc(t*sizeof(void*));
		if(!v)
			exit(-1);
		memcpy(v, list->pointers, (list->array_size*sizeof(void*)));
		free(list->pointers);
		list->pointers=v;
		list->array_size=t;
	}

	return(0);
}

